#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2018. All Rights Reserved.

"""Action Module server

This uses the `Circuits <http://circuitsframework.com/>` framework to
listen on multiple message destinations, send their events to the relevant
handlers, and acknowledge the messages when they have been processed.

"""

from __future__ import print_function

import logging
from logging.handlers import RotatingFileHandler
import os
import filelock
from circuits import Manager, BaseComponent, Component, Debugger, Loader
import resilient
from resilient import parse_parameters
from resilient_circuits.component_loader import ComponentLoader
from resilient_circuits.actions_component import Actions, ResilientComponent
import resilient_circuits.keyring_arguments as keyring_arguments
from six import string_types
import re

APP_LOG_DIR = os.environ.get("APP_LOG_DIR", "logs")
PASSWD_PATTERNS = ['passcode','password','passwd','secret','pin']

application = None
logging_initialized = False

class RedactingFilter(logging.Filter):
    """ Redacting logging filter to prevent Resilient circuits sensitive password values from being logged.

    """
    def __init__(self):
        super(RedactingFilter, self).__init__()

    def filter(self, record):
        # Best effort regex filter pattern to redact password logging.
        if isinstance(record.msg, string_types):
            for p in PASSWD_PATTERNS:
                if p in record.msg.lower():
                    record.msg = re.sub(r"(?i)^(.*)({})(.*?:\s*)(.+?)((?:[\s,].*)*)$".format(p),
                                        r"\1\2\3***\5", record.msg)
        return True


class AppArgumentParser(keyring_arguments.ArgumentParser):
    """Helper to parse command line arguments."""
    DEFAULT_STOMP_PORT = 65001
    DEFAULT_COMPONENTS_DIR = ''
    DEFAULT_LOG_LEVEL = 'INFO'
    DEFAULT_LOG_FILE = 'app.log'
    DEFAULT_NO_PROMPT_PASS = "False"

    def __init__(self, config_file=None):

        # Temporary logging handler until the real one is created later
        temp_handler = logging.StreamHandler()
        temp_handler.setFormatter(logging.Formatter('%(asctime)s %(levelname)s [%(module)s] %(message)s'))
        temp_handler.setLevel(logging.INFO)
        logging.getLogger().addHandler(temp_handler)
        config_file = config_file or resilient.get_config_file()
        super(AppArgumentParser, self).__init__(config_file=config_file)

        default_components_dir = self.getopt("resilient", "componentsdir") or self.DEFAULT_COMPONENTS_DIR
        default_noload = self.getopt("resilient", "noload") or ""
        default_log_dir = self.getopt("resilient", "logdir") or APP_LOG_DIR
        default_log_level = self.getopt("resilient", "loglevel") or self.DEFAULT_LOG_LEVEL
        default_log_file = self.getopt("resilient", "logfile") or self.DEFAULT_LOG_FILE

        # STOMP port is usually 65001
        default_stomp_port = self.getopt("resilient", "stomp_port") or self.DEFAULT_STOMP_PORT
        # For some environments the STOMP TLS certificate will be different from the REST API cert
        default_stomp_cafile = self.getopt("resilient", "stomp_cafile") or None

        default_no_prompt_password = self.getopt("resilient",
                                                 "no_prompt_password") or self.DEFAULT_NO_PROMPT_PASS
        default_no_prompt_password = self._is_true(default_no_prompt_password)

        default_test_actions = self._is_true(self.getopt("resilient",
                                                         "test_actions")) or False
        default_resilient_mock = self.getopt("resilient",
                                             "resilient_mock") or None
        default_test_host = self.getopt("resilient", "test_host") or None
        default_test_port = self.getopt("resilient", "test_port") or None
        default_log_responses = self.getopt("resilient",
                                            "log_http_responses") or ""
        logging.getLogger().removeHandler(temp_handler)

        self.add_argument("--stomp-port",
                          type=int,
                          default=default_stomp_port,
                          help="Resilient server STOMP port number")
        self.add_argument("--stomp-cafile",
                          type=str,
                          action="store",
                          default=default_stomp_cafile,
                          help="Resilient server STOMP TLS certificate")
        self.add_argument("--componentsdir",
                          type=str,
                          default=default_components_dir,
                          help="Circuits components auto-load directory")
        self.add_argument("--noload",
                          type=str,
                          default=default_noload,
                          help="List of components that should not be loaded")
        self.add_argument("--logdir",
                          type=str,
                          default=default_log_dir,
                          help="Directory for log files")
        self.add_argument("--loglevel",
                          type=str,
                          default=default_log_level,
                          help="Log level")
        self.add_argument("--logfile",
                          type=str,
                          default=default_log_file,
                          help="File to log to")
        self.add_argument("--no-prompt-password",
                          type=bool,
                          default=default_no_prompt_password,
                          help="Never prompt for password on stdin")
        self.add_argument("--test-actions",
                          action="store_true",
                          default=default_test_actions,
                          help="Enable submitting test actions?")
        self.add_argument("--resilient-mock",
                          type=str,
                          action="store",
                          default=default_resilient_mock,
                          help=("Mock class defintion. "
                                "(lib/my_mock_file.MyMockClass)"))
        self.add_argument("--test-host",
                          type=str,
                          action="store",
                          default=default_test_host,
                          help=("For use with --test-actions option. "
                                "Host or IP to bind test server to."))
        self.add_argument("--test-port",
                          type=int,
                          action="store",
                          default=default_test_port,
                          help=("For use with --test-actions option. "
                                "Port to bind test server to."))
        self.add_argument("--log-http-responses",
                          type=str,
                          default=default_log_responses,
                          help=("Log all responses from Resilient "
                                "REST API to this directory"))

    def parse_args(self, args=None, namespace=None):
        """Parse commandline arguments and construct an opts dictionary"""
        opts = super(AppArgumentParser, self).parse_args(args, namespace)
        if self.config:
            for section in self.config.sections():
                items = dict((item.lower(), self.config.get(section, item)) for item in self.config.options(section))
                opts.update({section: items})

            parse_parameters(opts)
        return opts

    @staticmethod
    def _is_true(value):
        if value:
            return value.lower()[0] in ("1", "t", "y")
        else:
            return False


# Main component for our application
class App(Component):
    """Our main app component, which sets up the Resilient services and other components"""

    FILE_LOG_FORMAT = '%(asctime)s %(levelname)s [%(module)s] %(message)s'
    SYSLOG_LOG_FORMAT = '%(module)s: %(levelname)s %(message)s'
    STDERR_LOG_FORMAT = '%(asctime)s %(levelname)s [%(module)s] %(message)s'

    def __init__(self, auto_load_components=True, config_file=None):
        super(App, self).__init__()
        # Read the configuration options
        self.action_component = None
        self.component_loader = None
        self.auto_load_components = auto_load_components
        self.config_file = config_file or resilient.get_config_file()
        self.do_initialization()

    def do_initialization(self):
        self.opts = AppArgumentParser(config_file=self.config_file).parse_args()

        self.config_logging(self.opts["logdir"], self.opts["loglevel"], self.opts['logfile'])
        LOG.info("Configuration file: %s", self.config_file)
        LOG.info("Resilient server: %s", self.opts.get("host"))
        LOG.info("Resilient user: %s", self.opts.get("email"))
        LOG.info("Resilient org: %s", self.opts.get("org"))
        LOG.info("Logging Level: %s", self.opts.get("loglevel"))
        if self.opts.get("test_actions", False):
            # Make all components aware that we are in test mode
            ResilientComponent.test_mode = True

        # Connect to events from Action Module.
        # Note: this must be done before components are loaded, because it uses
        # each component's "channel" to initiate subscription to the message queue.
        if self.opts.get("resilient", None) and self.opts["resilient"].get("actions_component", None):
            # user specified actions_component to use
            path, filename = os.path.split(self.opts["resilient"]["actions_component"])
            LOG.info("User specified actions_component: {}, from {}".format(filename, path))
            loader = Loader(init_kwargs={"opts": self.opts},
                            paths=[path])
            self.action_component = loader.load(filename)
        else:
            self.action_component = Actions(self.opts)
        self.action_component.register(self)

        # Register a `loader` to dynamically load
        # all Circuits components in the 'componentsdir' directory
        if self.auto_load_components:
            LOG.info("Components auto-load directory: %s",
                     self.opts["componentsdir"] or "(none)")
            if not self.component_loader:
                self.component_loader = ComponentLoader(self.opts)
            else:
                LOG.info("Updating and re-registering ComponentLoader")
                self.component_loader.opts = self.opts
            self.component_loader.register(self)

    def config_logging(self, logdir, loglevel, logfile):
        """ set up some logging """
        global LOG_PATH, LOG, logging_initialized

        LOG_PATH = os.path.join(logdir, logfile)
        LOG_PATH = os.path.expanduser(LOG_PATH)
        LOG = logging.getLogger(__name__)

        # Only do this once! (mostly for pytest)
        if logging_initialized:
            LOG.info("Logging already initialized.")
            return
        logging_initialized = True

        # Ignore syslog errors from message-too-long
        logging.raiseExceptions = False

        try:
            numeric_level = getattr(logging, loglevel)
            logging.getLogger().setLevel(numeric_level)
        except AttributeError as e:
            logging.getLogger().setLevel(logging.INFO)
            LOG.warning("Invalid logging level specified. Using INFO level")

        if logging.getLogger().getEffectiveLevel() == logging.DEBUG:
            self += Debugger()

        file_handler = RotatingFileHandler(LOG_PATH, maxBytes=10000000,
                                           backupCount=10)
        file_handler.setFormatter(logging.Formatter(self.FILE_LOG_FORMAT))
        logging.getLogger().addHandler(file_handler)
        syslog = logging.handlers.SysLogHandler()
        syslog.setFormatter(logging.Formatter(self.SYSLOG_LOG_FORMAT))
        logging.getLogger().addHandler(syslog)
        stderr = logging.StreamHandler()
        stderr.setFormatter(logging.Formatter(self.STDERR_LOG_FORMAT))
        logging.getLogger().addHandler(stderr)
        # Add password redacting filter for logging.
        syslog.addFilter(RedactingFilter())
        file_handler.addFilter(RedactingFilter())
        stderr.addFilter(RedactingFilter())

    def load_all_success(self, event):
        """OK, component loader says we're ready"""
        LOG.info("Components loaded")

        # For debugging, print out the tree of all loaded components
        def walk(depth, component):
            yield (u"  " * depth) + repr(component)
            if isinstance(component, BaseComponent) and callable(component.events):
                for event in component.events():
                    channels = ", ".join([h.channel or '*' for h in list(component._handlers[event])])
                    yield u"{}{}/{}".format((u"  " * (depth+1)), event, channels)
            if isinstance(component, Manager):
                for c in component.components:
                    for thing in walk(depth+1, c):
                        yield thing
        tree = walk(1, self.root)
        LOG.debug(u"Components:\n" + ("\n".join(tree)))

    def load_all_failure(self, event):
        """OK, component loader says we're unable to start"""
        LOG.error("A component failed to load.  The application cannot start.")
        self.stop()

    def reload_opts(self):
        """Reload the configuration options in case they changed"""
        LOG.debug("Reload opts")
        self.opts.update(AppArgumentParser().parse_args())

    def started(self, event, component):
        """Started Event Handler"""
        LOG.info("App Started")

    def stopped(self, event, component):
        """Stopped Event Handler"""
        LOG.info("App Stopped")


def get_lock():
    """Create a filelock"""

    # The run() method uses a file lock in the user's ~/.resilient directory to prevent multiple instances
    # of resilient circuits running.  You can override the lockfile name in the
    # (and so allow multiple) by setting APP_LOCK_FILE in the environment.
    app_lock_file = os.environ.get("APP_LOCK_FILE", "")

    if not app_lock_file:
        lockfile = os.path.expanduser(os.path.join("~", ".resilient", "resilient_circuits_lockfile"))
        resilient_dir = os.path.dirname(lockfile)
        if not os.path.exists(resilient_dir):
            os.makedirs(resilient_dir)
    else:
        lockfile = os.path.expanduser(app_lock_file)
    lock = filelock.FileLock(lockfile)
    return lock


def run(*args, **kwargs):
    """Main app"""

    # define lock
    # this prevents multiple, identical circuits from running at the same time
    lock = get_lock()

    # The main app component initializes the Resilient services
    global application
    try:
        # attempt to lock file, wait 1 second for lock
        with lock.acquire(timeout=1):
            assert lock.is_locked
            application = App(*args, **kwargs)
            application.run()

    except filelock.Timeout:
        # file is probably already locked
        print("Failed to acquire lock on {0} - "
              "you may have another instance of Resilient Circuits running".format(os.path.abspath(lock.lock_file)))
    except OSError as exc:
        # Some other problem accessing the lockfile
        print("Unable to lock {0}: {1}".format(os.path.abspath(lock.lock_file), exc))
    # finally:
    #    LOG.info("App finished.")


if __name__ == "__main__":
    run()



#!/usr/bin/env python
# (c) Copyright IBM Corp. 2010, 2019. All Rights Reserved.

""" Python Module that handles the Packaging of a Resilient Extention
from source code and the Converting of an (old) Integration into a
Resilient Extension """

from resilient_circuits.util.ext.ExtPackage import ExtPackage
from resilient_circuits.util.ext.ExtConvert import ExtConvert
from resilient_circuits.util.ext.ExtException import ExtException



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2019. All Rights Reserved.
# pylint: disable=line-too-long

"""Python Module to handle resilient-circuits ext: commands"""

import logging
import os
from resilient_circuits.util.ext import ExtPackage, ExtConvert, ExtException

# Get the same logger object that is used in resilient_circuits_cmd.py
LOG = logging.getLogger("resilient_circuits_cmd_logger")

SUPPORTED_EXT_COMMANDS = (
    "ext:package",
    "ext:convert")


def ext_command_handler(cmd, args):
    """Function that handles all resilient-circuits commands prefixed with 'ext:'"""

    # Handle if an unsupported command was called
    if cmd not in SUPPORTED_EXT_COMMANDS:
        raise ExtException("Unsupported command: {0}. Supported commands are: {1}".format(cmd, SUPPORTED_EXT_COMMANDS))

    path_to_src = None
    display_name = None
    keep_build_dir = False

    # Handle arguments
    if hasattr(args, "p"):
        path_to_src = os.path.abspath(args.p)

    if hasattr(args, "display_name"):
        display_name = args.display_name

    if hasattr(args, "keep_build_dir"):
        keep_build_dir = args.keep_build_dir

    LOG.debug("-----START ARGS DEBUG------")
    LOG.debug("args:\t\t\t%s", args)
    LOG.debug("path_to_src:\t\t%s", path_to_src)
    LOG.debug("display_name:\t\t%s", display_name)
    LOG.debug("keep_build_dir:\t\t%s", keep_build_dir)
    LOG.debug("-----END ARGS DEBUG-------")

    # Handle "ext:package"
    if cmd == SUPPORTED_EXT_COMMANDS[0]:
        ext_creator = ExtPackage(cmd)
        ext_creator.package_extension(path_to_src, custom_display_name=display_name, keep_build_dir=keep_build_dir)

    # Handle "ext:convert"
    elif cmd == SUPPORTED_EXT_COMMANDS[1]:
        ext_creator = ExtConvert(cmd)
        ext_creator.convert_to_extension(path_to_src, custom_display_name=display_name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2019. All Rights Reserved.
# pylint: disable=line-too-long

""" Python Module that exposes the ExtConvert class """

import logging
import os
import tempfile
import shutil
import zipfile
import tarfile
from resilient_circuits.util.ext.ExtCreate import ExtCreate
from resilient_circuits.util.ext.ExtException import ExtException

# Get the same logger object that is used in resilient_circuits_cmd.py
LOG = logging.getLogger("resilient_circuits_cmd_logger")


class ExtConvert(ExtCreate):
    """ ExtConvert is a subclass of ExtCreate. It exposes one
    method: package_extension() """

    @staticmethod
    def __get_tar_file_path_to_extract__(tar_members, file_name):
        """Loop all the tar_members and return the path to the member that matcheds file_name.
        Raise an Exception if cannot find file_name in the tar package"""

        for member in tar_members:
            tar_file_name = os.path.split(member.name)

            if tar_file_name[1] == file_name:
                return member.name

        raise ExtException("Invalid built distribution. Could not find {0}".format(file_name))

    @staticmethod
    def __extract_file_from_tar__(filename_to_extract, tar_file, output_dir):
        """Extract the given filename to the output_dir from the given tar_file
        and return the path to the extracted file"""

        tar_members = tar_file.getmembers()

        tar_path = ExtConvert.__get_tar_file_path_to_extract__(tar_members, filename_to_extract)
        tar_file.extract(member=tar_path, path=output_dir)

        return os.path.join(output_dir, tar_path)

    @staticmethod
    def __get_required_files_from_tar_file__(path_tar_file, dict_required_files, output_dir):
        """Loop the keys of dict_required_files (which will be file names).
        Extract each file and get the path to the extracted file.
        Return a dict of each file name and its extracted path"""

        return_dict = {}

        with tarfile.open(name=path_tar_file, mode="r") as tar_file:

            for file_name in dict_required_files.keys():
                return_dict[file_name] = ExtConvert.__extract_file_from_tar__(file_name, tar_file, output_dir)

        return return_dict

    @classmethod
    def convert_to_extension(cls, path_built_distribution, custom_display_name=None):
        """ Function that converts an (old) Integration into a Resilient Extension.
        Validates then converts the given built_distribution (either .tar.gz or .zip).
        Returns the path to the new Extension.zip
        - path_built_distribution [String]:
            - If a .tar.gz: must include a setup.py, customize.py and config.py file.
            - If a .zip: must include a valid .tar.gz.
        - custom_display_name [String]: will give the Extension that display name. Default: name from setup.py file
        - The Extension.zip will be produced in the same directory as path_built_distribution"""

        LOG.info("Converting extension from: %s", path_built_distribution)

        path_tmp_built_distribution, path_extracted_tar = None, None

        # Dict of the required files we need to try extract in order to create an Extension
        extracted_required_files = {
            "setup.py": None,
            "customize.py": None,
            "config.py": None
        }

        # Raise Exception if the user tries to pass a Directory
        if os.path.isdir(path_built_distribution):
            raise ExtException("You must specify a Built Distribution. Not a Directory\nDirectory Specified: {0}".format(path_built_distribution))

        # Raise Exception if not a .tar.gz or .zip file
        if not os.path.isfile(path_built_distribution) or (not tarfile.is_tarfile(path_built_distribution) and not zipfile.is_zipfile(path_built_distribution)):
            raise ExtException("File corrupt. Supported Built Distributions are .tar.gz and .zip\nInvalid Built Distribution provided: {0}".format(path_built_distribution))

        # Validate we can read the built distribution
        cls.__validate_file_paths__(os.R_OK, path_built_distribution)

        # Create a tmp directory
        path_tmp_dir = tempfile.mkdtemp(prefix="resilient-circuits-tmp-")

        try:
            # Copy built distribution to tmp directory
            shutil.copy(path_built_distribution, path_tmp_dir)

            # Get the path of the built distribution in the tmp directory
            path_tmp_built_distribution = os.path.join(path_tmp_dir, os.path.split(path_built_distribution)[1])

            # Handle if it is a .tar.gz file
            if tarfile.is_tarfile(path_tmp_built_distribution):

                LOG.info("A .tar.gz file was provided. Will now attempt to convert it to a Resilient Extension.")

                # Extract the required files to the tmp dir and return a dict of their paths
                extracted_required_files = cls.__get_required_files_from_tar_file__(
                    path_tar_file=path_tmp_built_distribution,
                    dict_required_files=extracted_required_files,
                    output_dir=path_tmp_dir)

                path_extracted_tar = path_tmp_built_distribution

            # Handle if is a .zip file
            elif zipfile.is_zipfile(path_tmp_built_distribution):

                LOG.info("A .zip file was provided. Will now attempt to convert it to a Resilient Extension.")

                with zipfile.ZipFile(file=path_tmp_built_distribution, mode="r") as zip_file:

                    # Get a List of all the members of the zip file (including files in directories)
                    zip_file_members = zip_file.infolist()

                    LOG.info("\nValidating Built Distribution: %s", path_built_distribution)

                    # Loop the members
                    for zip_member in zip_file_members:

                        LOG.info("\t- %s", zip_member.filename)

                        # Extract the member
                        path_extracted_member = zip_file.extract(member=zip_member, path=path_tmp_dir)

                        # Handle if the member is a directory
                        if os.path.isdir(path_extracted_member):

                            LOG.debug("\t\t- Is a directory.\n\t\t- Skipping...")

                            # delete the extracted member
                            shutil.rmtree(path_extracted_member)
                            continue

                        # Handle if it is a .tar.gz file
                        elif tarfile.is_tarfile(path_extracted_member):

                            LOG.info("\t\t- Is a .tar.gz file!")

                            # Set the path to the extracted .tar.gz file
                            path_extracted_tar = path_extracted_member

                            # Try to extract the required files from the .tar.gz
                            try:
                                extracted_required_files = cls.__get_required_files_from_tar_file__(
                                    path_tar_file=path_extracted_member,
                                    dict_required_files=extracted_required_files,
                                    output_dir=path_tmp_dir)

                                LOG.info("\t\t- Found files: %s\n\t\t- Its path: %s\n\t\t- Is a valid Built Distribution!", ", ".join(extracted_required_files.keys()), path_extracted_tar)
                                break

                            except ExtException as err:
                                # If "invalid" is in the error message,
                                # then we did not find one of the required files in the .tar.gz
                                # so we warn the user, delete the extracted member and continue the loop
                                if "invalid" in err.message.lower():
                                    LOG.warning("\t\t- Failed to extract required files: %s\n\t\t- Invalid format.\n%s", ", ".join(extracted_required_files.keys()), err.message)
                                    os.remove(path_extracted_member)
                                else:
                                    raise ExtException(err)

                        # Handle if it is a regular file
                        elif os.path.isfile(path_extracted_member):

                            # Get the file name
                            file_name = os.path.basename(path_extracted_member)

                            # If the file is a required one, add its path to the dict
                            if file_name in extracted_required_files:
                                LOG.info("\t\t- Found %s file", file_name)
                                extracted_required_files[file_name] = path_extracted_member

                                # Set the path to extracted tar to this zip file
                                path_extracted_tar = zip_file.filename

                            # Else its some other file, so skip
                            else:
                                LOG.debug("\t\t- It is not a .tar.gz file\n\t\t- Skipping...")
                                os.remove(path_extracted_member)

                        # if extracted_required_files contains values for all required files, then break
                        if all(extracted_required_files.values()):
                            LOG.info("\t\t- This is a valid Built Distribution!")
                            break

                        else:
                            LOG.debug("\t\t- Is not a valid .tar.gz built distribution\n\t\t- Skipping...")

            # If we could not get all the required files to create an Extension, raise an error
            if not all(extracted_required_files.values()):
                raise ExtException("Could not extract required files from given Built Distribution\nRequired Files: {0}\nDistribution: {1}".format(
                    ", ".join(extracted_required_files.keys()), path_built_distribution))

            # Create the extension
            path_tmp_the_extension_zip = cls.create_extension(
                path_setup_py_file=extracted_required_files.get("setup.py"),
                path_customize_py_file=extracted_required_files.get("customize.py"),
                path_config_py_file=extracted_required_files.get("config.py"),
                output_dir=path_tmp_dir,
                path_built_distribution=path_extracted_tar,
                custom_display_name=custom_display_name
            )

            # Copy the extension.zip to the same directory as the original built distribution
            shutil.copy(path_tmp_the_extension_zip, os.path.dirname(path_built_distribution))

            # Get the path to the final extension.zip
            path_the_extension_zip = os.path.join(os.path.dirname(path_built_distribution), os.path.basename(path_tmp_the_extension_zip))

            LOG.info("Extension created at: %s", path_the_extension_zip)

            return path_the_extension_zip

        except Exception as err:
            raise ExtException(err)

        finally:
            # Remove the tmp directory
            shutil.rmtree(path_tmp_dir)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2019. All Rights Reserved.

"""Global accessor for the Resilient REST API"""

import resilient

resilient_client = None
connection_opts = None


def reset_resilient_client():
    """Reset the cached client"""
    global resilient_client
    resilient_client = None


def get_resilient_client(opts):
    """Get a connected instance of SimpleClient for Resilient REST API"""
    global resilient_client
    global connection_opts

    new_opts = (opts.get("cafile"),
                opts.get("org"),
                opts.get("host"),
                opts.get("port"),
                opts.get("proxy_host"),
                opts.get("proxy_port"),
                opts.get("proxy_user"),
                opts.get("proxy_password"),
                opts.get("email"),
                opts.get("api_key_id"))
    if new_opts != connection_opts:
        resilient_client = None
        connection_opts = new_opts
    if resilient_client:
        return resilient_client

    resilient_client = resilient.get_client(opts)
    return resilient_client



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2018. All Rights Reserved.

"""Template processing"""

# Jinja template functions

from __future__ import print_function
import sys
import jinja2
import json
import pprint
import logging
import time
import calendar
import datetime
import pytz
import random
import re

try:
    # Python 3.2 adds html.escape() and deprecates cgi.escape().
    from html import escape as html_escape
except ImportError:
    from cgi import escape as html_escape

try:
    # Python 3.x
    from base64 import encodebytes as b64encode
except ImportError:
    from base64 import encodestring as b64encode

import urllib

LOG = logging.getLogger(__name__)


def js_filter(val):
    """Jinja2 filter function 'js' produces JSONified string of the value, without surrounding quotes"""
    if val is None or isinstance(val, jinja2.Undefined):
        return "null"
    js = json_filter(val)
    return js[1:-1]


def json_filter(val, indent=0):
    """Jinja2 filter function 'json' produces JSONified string of the value"""
    if val is None or isinstance(val, jinja2.Undefined):
        return "null"
    return json.dumps(val, indent=indent, sort_keys=True)


def html_filter(val):
    """Jinja2 filter function 'html' produces HTML-encoded string of the value"""
    if isinstance(val, jinja2.Undefined):
        return "[undefined]"
    return html_escape(val)


def url_filter(val):
    """Jinja2 filter function 'url' produces URL-encoded string of the value"""
    if isinstance(val, jinja2.Undefined):
        return "[undefined]"
    return urllib.quote(str(val))


def idna_filter(val):
    """Jinja2 filter function 'idna' encodes the value per RFC 3490"""
    if isinstance(val, jinja2.Undefined):
        return "[undefined]"
    return val.encode("idna").decode("utf-8")


def punycode_filter(val):
    """Jinja2 filter function 'punycode' encodes the value per RFC 3492"""
    if isinstance(val, jinja2.Undefined):
        return "[undefined]"
    return val.encode("punycode").decode("utf-8")


def ldap_filter(val):
    """Jinja2 filter function 'ldap' produces LDAP-encoded string of the value"""
    if isinstance(val, jinja2.Undefined):
        return "[undefined]"
    escaped = []
    for char in str(val):
        if char < '0' or char > 'z' or char in "\\*()":
            char = "\\%02x" % ord(char)
        escaped.append(char)
    return ''.join(escaped)


def ps_filter(val):
    """Jinja2 filter function 'ps' escapes for use in a PowerShell commandline"""
    if isinstance(val, jinja2.Undefined):
        return "[undefined]"
    escaped = []
    for char in str(val):
        if char in "`$#'\"":
            char = "`" + char
        elif char == '\0':
            char = "`0"
        elif char == '\a':
            char = "`a"
        elif char == '\b':
            char = "`b"
        elif char == '\f':
            char = "`f"
        elif char == '\n':
            char = "`n"
        elif char == '\r':
            char = "`r"
        elif char == '\t':
            char = "`t"
        elif char == '\v':
            char = "`v"
        escaped.append(char)
    return ''.join(escaped)


def sh_filter(val):
    """Jinja2 filter function 'sh' escapes for use in a Unix shell commandline"""
    if isinstance(val, jinja2.Undefined):
        return "[undefined]"
    escaped = []
    for char in str(val):
        if char in "$#\"":
            char = "\\" + char
        elif ord(char) < 32 or ord(char) > 126:
            char = "\\%03o" % ord(char)
        escaped.append(char)
    return ''.join(escaped)


def pretty_filter(val, indent=2):
    """Jinja2 filter function 'pretty' produces pretty-printed string of the value"""
    if isinstance(val, jinja2.Undefined):
        return "[undefined]"

    def nice_repr(object, context, maxlevels, level):
        if sys.version_info.major < 3:
            typ = type(object)
            if typ is unicode:
                object = object.encode("utf-8")
        return pprint._safe_repr(object, context, maxlevels, level)

    printer = pprint.PrettyPrinter(indent=indent)
    printer.format = nice_repr
    return printer.pformat(val)


def iso8601(val):
    """Assuming val is an epoch milliseconds timestamp, produce ISO8601 datetime"""
    dt = datetime.datetime.utcfromtimestamp(int(int(val)/1000))
    return pytz.UTC.localize(dt).isoformat()


def timestamp(val):
    """Try convert non-timestamp values to a timestamp

       >>> timestamp({"year": 2018, "month": 8, "day": 1, "timezoneID": "CET"})
       1533078000000

       >>> timestamp(jinja2.Undefined())
       'null'

       >>> timestamp("now") > 1530000000000
       True

       >>> timestamp("now") > 2000000000000 # 2033
       False
    """
    if isinstance(val, dict):
        y = val.get("year", 1970)
        m = val.get("month", 1)
        d = val.get("day", 1)
        h = val.get("hour", 0)
        n = val.get("minute", 0)
        s = val.get("second", 0)
        u = val.get("milliSecond", 0)
        z = pytz.timezone(val.get("timezoneID", "UTC"))
        dt = datetime.datetime(y, m, d, h, n, s, u, z)
        return int(calendar.timegm(dt.utctimetuple()) * 1000)
    if isinstance(val, jinja2.Undefined):
        return "null"
    if isinstance(val, datetime.datetime):
        return int(calendar.timegm(val.utctimetuple()) * 1000)
    if val == "now":
        return int(calendar.timegm(datetime.datetime.now().utctimetuple()) * 1000)
    return val


def uniq(val, key=None):
    """Produce the unique list.  If val is a dict, produce unique of key values.

       >>> sorted(uniq([1,2,3,2]))
       [1, 2, 3]

       >>> sorted(uniq([ {"a":1}, {"a":2}, {"a":3}, {"a":2}]))
       [{'a': 1}, {'a': 2}, {'a': 3}]

       >>> sorted(uniq([ {"a":1}, {"a":2}, {"a":3}, {"a":2}, Exception()], "a"))
       [{'a': 1}, {'a': 2}, {'a': 3}, Exception()]
    """
    if not isinstance(val, list):
        return val
    if key is None:
        try:
            return list(set(val))
        except TypeError:
            pass
    keys = []
    values = []
    for value in val:
        try:
            thiskey = value[key]
        except:
            thiskey = repr(value)
        if thiskey not in keys:
            keys.append(thiskey)
            values.append(value)
    return values


def sample_filter(val, count=None):
    """Return a random sample from a list"""
    if count is None:
        # Return a single value
        try:
            return random.sample(list(val), 1)[0]
        except ValueError:
            return None
    else:
        # Return a list
        try:
            return random.sample(list(val), count)
        except ValueError:
            return []


def camel_filter(val):
    """Return CamelCase

       >>> camel_filter("a#bc_def")
       'ABcDef'
    """
    titlecase = val.title()
    return re.sub(r"[\W^_]", "", titlecase)


def base64_filter(val, indent=2):
    """Jinja2 filter function 'base64' breaks text into fixed-width blocks"""
    if isinstance(val, jinja2.Undefined):
        return ""
    s = json.dumps(val).encode("utf-8")
    return b64encode(s).decode("utf-8")


JINJA_FILTERS = {"json": json_filter,
                 "js": js_filter,
                 "html": html_filter,
                 "url": url_filter,
                 "idna": idna_filter,
                 "punycode": punycode_filter,
                 "ldap": ldap_filter,
                 "ps": ps_filter,
                 "sh": sh_filter,
                 "pretty": pretty_filter,
                 "timestamp": timestamp,
                 "iso8601": iso8601,
                 "uniq": uniq,
                 "sample": sample_filter,
                 "camel": camel_filter,
                 "base64": base64_filter}


# Maintain one global Environment
ENV = jinja2.Environment(autoescape=jinja2.select_autoescape(default_for_string=False))
ENV.globals.update(JINJA_FILTERS)
ENV.filters.update(JINJA_FILTERS)


def render(template, data):
    """Render data into a template, producing a string result

        The template is usually a string, but can be a dict.

        >>> render("template {{value}}", {"value":"123"})
        u'template 123'

        >>> render({"template": "{{value}}"}, {"value":"123"})
        u'{"template": "123"}'

        You can escape values using the 'json' filter,
        or the 'url' or 'html' or 'ldap' filters.

        >>> render('{"template": {{value|json}} }', {"value":'1"23'})
        u'{"template": "1\\\\"23" }'

        >>> render('{"template": "{{value|js}}" }', {"value":'1"23'})
        u'{"template": "1\\\\"23" }'

        >>> render('{"template": {{value|ldap}} }', {"value":'1*23'})
        u'{"template": 1\\\\2a23 }'

        >>> render('shell "{{value|ps}}"', {"value":'$"foo"'})
        u'shell "`$`"foo`""'

        >>> render('shell "{{value|sh}}"', {"value":'$"foo"'})
        u'shell "\\\\$\\\\"foo\\\\""'

        >>> render('template={{value|timestamp}}', {"value":0})
        u'template=0'

        >>> render('template={{value|timestamp}}', {})
        u'template=null'

        >>> render('template={{value|timestamp}}', {"value":{"year":2015, "month":7, "day":15}})
        u'template=1436918400000'

        >>> render('template={{value|timestamp}}', {"value":datetime.datetime(2015, 7, 15)})
        u'template=1436918400000'

        >>> d = json.loads('{"attributes": \
                {"cn": ["Albert Einstein"], "createTimestamp": "2014-02-21 16:51:33+00:00", \
                 "creatorsName": "cn=admin,dc=example,dc=com", \
                 "entryCSN": ["20150720185447.990131Z#000000#000#000000"], \
                 "entryDN": "uid=einstein,dc=example,dc=com", \
                 "entryUUID": "29f6dc28-2f64-1033-898b-a53eb149a944", \
                 "hasSubordinates": false, \
                 "mail": ["einstein@ldap.forumsys.com"], \
                 "modifiersName": "cn=admin,dc=example,dc=com", \
                 "modifyTimestamp": "2015-07-20 18:54:47+00:00", \
                 "objectClass": ["inetOrgPerson", "organizationalPerson", "person", "top"], \
                 "sn": ["Einstein"], \
                 "structuralObjectClass": "inetOrgPerson", \
                 "subschemaSubentry": "cn=Subschema", \
                 "telephoneNumber": ["314-159-2653"], \
                 "uid": ["einstein"]}, \
                "dn": "uid=einstein,dc=example,dc=com"}')
        >>> render('{"description": "DN={{dn}}, mail={{attributes.mail[0]}}"}',d)
        u'{"description": "DN=uid=einstein,dc=example,dc=com, mail=einstein@ldap.forumsys.com"}'
    """

    stringtemplate = template
    if isinstance(template, dict):
        stringtemplate = json.dumps(template, sort_keys=True)

    try:
        jtemplate = ENV.from_string(stringtemplate)
    except jinja2.exceptions.TemplateSyntaxError:
        LOG.error("Render failed, with template: {0}".format(stringtemplate))
        raise

    try:
        stringvalue = jtemplate.render(data)
    except jinja2.exceptions.TemplateError:
        LOG.error("Render failed, with data: {0}".format(data))
        raise
    return stringvalue


def render_json(template, data):
    """Render data into a template, producing a JSON result
       Also clean up any "really bad" control characters to avoid failure.

       >>> d = {"value": "the" + chr(10) + "new" + chr(10) + "thing"}
       >>> render_json('{"result":"{{value}}"}', d)
       {u'result': u'the new thing'}

       >>> d = {"value": "the" + chr(1) + "new" + chr(9) + "thing"}
       >>> render_json('{"result":"{{value}}"}', d)
       {u'result': u'the new thing'}
    """
    result = render(template, data)
    for n in range(1, 32):
        result = result.replace(chr(n), " ")
    try:
        value = json.loads(result)
    except:
        LOG.info(result)
        raise
    return value


def environment():
    return ENV


def test(template):
    """Test some basic functionality

        The result is a string.  It's ok to not use any tags.
        >>> test("thing")
        u'thing'

        Tags return unescaped data
        >>> test("this is my {{number}}")
        u'this is my 42'
        >>> test("this is my {{string}}")
        u'this is my templates'

        The 'json' filter return JSON-quoted result,
        >>> test("this is my {{number|json}}")
        u'this is my 42'
        >>> test("this is my {{string|json}}")
        u'this is my "templates"'

        Dotted forms can navigate the tree of data,
        >>> test("object.name is {{object.name}}")
        u'object.name is v<a>lue'

        To render HTML, use the 'html' filter,
        >>> test( {"text":"<div>{{object.name|html}}</div>"} )
        u'{"text": "<div>v&lt;a&gt;lue</div>"}'

        Test the 'iso8601' filter
        >>> test("Go back to {{epochdate|iso8601}}")
        u'Go back to 2002-05-02T01:47:30+00:00'

        The filters can also be used as global functions json(), html() etc,
        >>> test('{{html("f<oo")}}')
        u'f&lt;oo'

        Index by index,
        >>> test("number is {{numbers[1]}}")
        u'number is 2'

        Loop over collections,
        >>> test("numbers are {% for n in numbers%}{{n}},{% endfor %}")
        u'numbers are 1,2,3,'

        Built-in filters should "just work",
        >>> test("{{string|upper}}")
        u'TEMPLATES'

        >>> test("{{domain|idna}}")
        u'xn--c1yn36f'

        >>> test("{{domain|punycode}}")
        u'c1yn36f'

    """
    data = {"string": "templates",
            "number": 42,
            "numbers": [1, 2, 3],
            "object": {"name": "v<a>lue"},
            "epochdate": 1020304050607,
            "domain": u"\u9ede\u770b"}
    return unicode(render(template, data))


def main():
    """Just some tests or usage statement"""
    import sys
    if len(sys.argv) > 1:
        test_template = sys.argv[1]
    else:
        import doctest
        doctest.testmod()
        test_template = {"usage": "Use {{string}} in json, or {{string|json}} within strings"}
    print(test_template)
    print(test(test_template))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2018. All Rights Reserved.

""" Circuits component for handling Stomp Connection """

import logging
import ssl
import time
import traceback
from circuits import BaseComponent, Timer
from circuits.core.handlers import handler
from stompest.config import StompConfig
from stompest.protocol import StompSpec, StompSession
from stompest.sync import Stomp
from stompest.error import StompConnectionError, StompError
from stompest.sync.client import LOG_CATEGORY
from resilient_circuits.stomp_events import *
from resilient_circuits.stomp_transport import EnhancedStompFrameTransport


StompSpec.DEFAULT_VERSION = '1.2'
ACK_CLIENT_INDIVIDUAL = StompSpec.ACK_CLIENT_INDIVIDUAL
ACK_AUTO = StompSpec.ACK_AUTO
ACK_CLIENT = StompSpec.ACK_CLIENT
ACK_MODES = (ACK_CLIENT_INDIVIDUAL, ACK_AUTO, ACK_CLIENT)

LOG = logging.getLogger(__name__)


class StompClient(BaseComponent):

    channel = "stomp"

    def init(self, host, port, username=None, password=None,
             connect_timeout=3, connected_timeout=3,
             version=StompSpec.VERSION_1_2, accept_versions=["1.0", "1.1", "1.2"],
             heartbeats=(0, 0), ssl_context=None,
             use_ssl=True,
             key_file=None,
             cert_file=None,
             ca_certs=None,
             ssl_version=ssl.PROTOCOL_SSLv23,  # means 'latest available'
             key_file_password=None,
             proxy_host=None,
             proxy_port=None,
             proxy_user=None,
             proxy_password=None,
             channel=channel):
        """ Initialize StompClient.  Called after __init__ """
        self.channel = channel
        if proxy_host:
            LOG.info("Connect to %s:%s through proxy %s:%d", host, port, proxy_host, proxy_port)
        else:
            LOG.info("Connect to %s:%s", host, port)

        if use_ssl and not ssl_context:

            ssl_params = dict(key_file=key_file,
                              cert_file=cert_file,
                              ca_certs=ca_certs,
                              ssl_version=ssl_version,
                              password=key_file_password)
            LOG.info("Request to use old-style socket wrapper: %s", ssl_params)
            ssl_context = ssl_params

        if use_ssl:
            uri = "ssl://%s:%s" % (host, port)
        else:
            uri = "tcp://%s:%s" % (host, port)

        # Configure failover options so it only tries to connect once
        self._stomp_server = "failover:(%s)?maxReconnectAttempts=1,startupMaxReconnectAttempts=1" % uri

        self._stomp_config = StompConfig(uri=self._stomp_server, sslContext=ssl_context,
                                         version=version,
                                         login=username,
                                         passcode=password)

        self._heartbeats = heartbeats
        self._accept_versions = accept_versions
        self._connect_timeout = connect_timeout
        self._connected_timeout = connected_timeout
        Stomp._transportFactory = EnhancedStompFrameTransport
        Stomp._transportFactory.proxy_host = proxy_host
        Stomp._transportFactory.proxy_port = proxy_port
        Stomp._transportFactory.proxy_user = proxy_user
        Stomp._transportFactory.proxy_password = proxy_password
        self._client = Stomp(self._stomp_config)
        self._subscribed = {}
        self.server_heartbeat = None
        self.client_heartbeat = None
        self.last_heartbeat = 0
        self.ALLOWANCE = 2  # multiplier for heartbeat timeouts

    @property
    def connected(self):
        if self._client.session:
            return self._client.session.state == StompSession.CONNECTED
        else:
            return False

    @property
    def socket_connected(self):
        try:
            if self._client._transport:
                return True
        except:
            pass
        return False

    @property
    def subscribed(self):
        return self._subscribed.keys()

    @property
    def stomp_logger(self):
        return LOG_CATEGORY

    @handler("Disconnect")
    def _disconnect(self, receipt=None, flush=True, reconnect=False):
        try:
            if flush:
                self._subscribed = {}
            if self.connected:
                self._client.disconnect(receipt=receipt)
        except Exception as e:
            LOG.error("Failed to disconnect client")
        try:
            self.fire(Disconnected(reconnect=reconnect))
            self._client.close(flush=flush)
        except Exception as e:
            LOG.error("Failed to close client connection")

        return "disconnected"

    def start_heartbeats(self):
        LOG.info("Client HB: %s  Server HB: %s", self._client.clientHeartBeat, self._client.serverHeartBeat)
        if self._client.clientHeartBeat:
            if self.client_heartbeat:
                # Timer already exists, just reset it
                self.client_heartbeat.reset()
            else:
                LOG.info("Client will send heartbeats to server")
                # Send heartbeats at 80% of agreed rate
                self.client_heartbeat = Timer((self._client.clientHeartBeat / 1000.0) * 0.8,
                                              ClientHeartbeat(), persist=True)
                self.client_heartbeat.register(self)
        else:
            LOG.info("No Client heartbeats will be sent")

        if self._client.serverHeartBeat:
            if self.server_heartbeat:
                # Timer already exists, just reset it
                self.server_heartbeat.reset()
            else:
                LOG.info("Requested heartbeats from server.")
                # Allow a grace period on server heartbeats
                self.server_heartbeat = Timer((self._client.serverHeartBeat / 1000.0) * self.ALLOWANCE,
                                              ServerHeartbeat(), persist=True)
                self.server_heartbeat.register(self)
        else:
            LOG.info("Expecting no heartbeats from Server")

    @handler("Connect")
    def connect(self, event, host=None, *args, **kwargs):
        """ connect to Stomp server """
        LOG.info("Connect to Stomp...")
        try:
            self._client.connect(heartBeats=self._heartbeats,
                                 host=host,
                                 versions=self._accept_versions,
                                 connectTimeout=self._connect_timeout,
                                 connectedTimeout=self._connected_timeout)
            LOG.debug("State after Connection Attempt: %s", self._client.session.state)
            if self.connected:
                LOG.info("Connected to %s", self._stomp_server)
                self.fire(Connected())
                self.start_heartbeats()
                return "success"

        except StompConnectionError as err:
            LOG.debug(traceback.format_exc())
            self.fire(ConnectionFailed(self._stomp_server))
            event.success = False
        return "fail"

    @handler("ServerHeartbeat")
    def check_server_heartbeat(self, event):
        """ Confirm that heartbeat from server hasn't timed out """
        now = time.time()
        self.last_heartbeat = self._client.lastReceived or self.last_heartbeat
        if self.last_heartbeat:
            elapsed = now-self.last_heartbeat
        else:
            elapsed = -1
        if ((self._client.serverHeartBeat / 1000.0) * self.ALLOWANCE + self.last_heartbeat) < now:
            LOG.error("Server heartbeat timeout. %d seconds since last heartbeat.", elapsed)
            event.success = False
            self.fire(HeartbeatTimeout())

    @handler("ClientHeartbeat")
    def send_heartbeat(self, event):
        if self.connected:
            LOG.debug("Sending heartbeat")
            try:
                self._client.beat()
            except (StompConnectionError, StompError) as err:
                event.success = False
                self.fire(OnStompError(None, err))

    @handler("generate_events")
    def generate_events(self, event):
        event.reduce_time_left(0.1)
        if not self.connected:
            return
        try:
            if self._client.canRead(0):
                frame = self._client.receiveFrame()
                LOG.debug("Recieved frame %s", frame)
                if frame.command == StompSpec.ERROR:
                    self.fire(OnStompError(frame, None))
                else:
                    self.fire(Message(frame))
        except (StompConnectionError, StompError) as err:
            LOG.error("Failed attempt to generate events.")
            self.fire(OnStompError(None, err))

    @handler("Send")
    def send(self, event, destination, body, headers=None, receipt=None):
        LOG.debug("send()")
        try:
            self._client.send(destination, body=body.encode('utf-8'), headers=headers, receipt=receipt)
            LOG.debug("Message sent")
        except (StompConnectionError, StompError) as err:
            LOG.error("Error sending frame")
            event.success = False
            self.fire(OnStompError(None, err))
            raise  # To fire Send_failure event

    @handler("Subscribe")
    def _subscribe(self, event, destination, additional_headers=None, ack=ACK_CLIENT_INDIVIDUAL):
        if ack not in ACK_MODES:
            raise ValueError("Invalid client ack mode specified")
        if destination in self._client.session._subscriptions:
            LOG.debug("Ignoring subscribe request to %s. Already subscribed.", destination)
        LOG.info("Subscribe to message destination %s", destination)
        try:
            headers = {StompSpec.ACK_HEADER: ack,
                       'id': destination}
            if additional_headers:
                headers.update(additional_headers)

            # Set ID to match destination name for easy reference later
            frame, token = self._client.subscribe(destination,
                                                  headers)
            self._subscribed[destination] = token
        except (StompConnectionError, StompError) as err:
            LOG.error("Failed to subscribe to queue.")
            event.success = False
            LOG.debug(traceback.format_exc())
            self.fire(OnStompError(None, err))

    @handler("Unsubscribe")
    def _unsubscribe(self, event, destination):
        if destination not in self._subscribed:
            LOG.error("Unsubscribe Request Ignored. Not subscribed to %s", destination)
            return
        try:
            token = self._subscribed.pop(destination)
            frame = self._client.unsubscribe(token)
            LOG.debug("Unsubscribed: %s", frame)
        except (StompConnectionError, StompError) as err:
            event.success = False
            LOG.error("Unsubscribe Failed.")
            self.fire(OnStompError(frame, err))

    @handler("Message")
    def on_message(self, event, headers, message):
        LOG.debug("Stomp message received")

    @handler("Ack")
    def ack_frame(self, event, frame):
        LOG.debug("ack_frame()")
        try:
            self._client.ack(frame)
            LOG.debug("Ack Sent")
        except (StompConnectionError, StompError) as err:
            LOG.error("Error sending ack")
            event.success = False
            self.fire(OnStompError(frame, err))
            raise  # To fire Ack_failure event

    def get_subscription(self, frame):
        """ Get subscription from frame """
        _, token = self._client.message(frame)
        return self._subscribed[token]



import unittest
import os
import sys
import shutil
import zipfile
import json
from resilient_circuits.util.ext.ExtCreate import PREFIX_EXTENSION_ZIP
from resilient_circuits.util.ext import ExtPackage, ExtException

# Import mock_data (need to add path to support relative imports in PY3)
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from ext_tests.mock_data import mock_data

path_this_dir = os.path.dirname(os.path.realpath(__file__))
path_temp_test_dir = os.path.join(path_this_dir, "test_temp")
path_fn_mock_integration = os.path.join(path_this_dir, "mock_data", "fn_mock_integration")
path_fn_mock_integration_dist_dir = os.path.join(path_this_dir, "mock_data", "fn_mock_integration", "dist")

path_mock_extension_json = os.path.join(path_this_dir, "mock_data", "ext-fn_mock_integration-1.0.0", "extension.json")
path_mock_extension_json_custom_display_name = os.path.join(path_this_dir, "mock_data", "ext-fn_mock_integration-1.0.0", "extension_custom_display_name.json")


def get_dict_from_json_file(file_path):
    dict_to_return = {}
    with open(file_path, 'r') as the_file:
            dict_to_return = json.load(the_file)
    return dict_to_return


class ExtPackageClassTestPackageExtension(unittest.TestCase):

    maxDiff = None

    def setUp(self):
        self.ext_creator = ExtPackage("ext:package")

        # Create temp dir
        os.makedirs(path_temp_test_dir)

    def tearDown(self):
        # Remove dist dir
        shutil.rmtree(path_fn_mock_integration_dist_dir, ignore_errors=True)

        # Remove temp dir
        shutil.rmtree(path_temp_test_dir, ignore_errors=True)

    def validate_extension_json(self, path_the_extension_zip, path_mock_extension_json):

        # Get the mock data
        mock_extension_json = get_dict_from_json_file(path_mock_extension_json)

        # Extract the zip
        with zipfile.ZipFile(file=path_the_extension_zip, mode="r") as zip_file:
            zip_file.extractall(path=path_temp_test_dir)

        # Get the extension_json
        extension_json = get_dict_from_json_file(os.path.join(path_temp_test_dir, "extension.json"))

        # Compare
        self.assertEqual(mock_extension_json, extension_json)

    def test_package_extension_no_params(self):
        # Generate names for .zip and .tar.gz files that will be created
        expected_extension_zip_name = "{0}{1}-{2}.zip".format(PREFIX_EXTENSION_ZIP, mock_data.MOCK_INTEGRATION_NAME, mock_data.MOCK_INTEGRATION_VERSION)
        expected_built_distribution_name = "{0}-{1}.tar.gz".format(mock_data.MOCK_INTEGRATION_NAME, mock_data.MOCK_INTEGRATION_VERSION)

        # List of files expected in the dist dir
        expected_dist_dir_contents = [
            expected_extension_zip_name,
            expected_built_distribution_name
        ]

        # Expected path to the created extension.zip
        expected_path_the_extension_zip = os.path.join(path_fn_mock_integration_dist_dir, expected_extension_zip_name)

        # Package the fn_mock_integration
        path_the_extension_zip = self.ext_creator.package_extension(path_fn_mock_integration)
        dist_dir_contents = os.listdir(path_fn_mock_integration_dist_dir)

        # Compare
        self.assertEqual(path_the_extension_zip, expected_path_the_extension_zip)
        self.assertEqual(dist_dir_contents, expected_dist_dir_contents)

        # Validate the extension.json
        self.validate_extension_json(path_the_extension_zip, path_mock_extension_json)

    def test_package_extension_keep_build_dir_true(self):
        # Package the fn_mock_integration
        self.ext_creator.package_extension(
            path_to_src=path_fn_mock_integration,
            keep_build_dir=True)

        # Check the build dir still exists
        self.assertTrue(os.path.exists(os.path.join(path_fn_mock_integration_dist_dir, "build")))

    def test_package_extension_keep_build_dir_false(self):
        # Package the fn_mock_integration
        self.ext_creator.package_extension(
            path_to_src=path_fn_mock_integration,
            keep_build_dir=False)

        # Check the build dir does not exist
        self.assertFalse(os.path.exists(os.path.join(path_fn_mock_integration_dist_dir, "build")))

    def test_package_extension_custom_display_name(self):
        # Package the fn_mock_integration
        path_the_extension_zip = self.ext_creator.package_extension(
            path_to_src=path_fn_mock_integration,
            custom_display_name="Mock Custom Display Name")

        # Validate the extension.json
        self.validate_extension_json(path_the_extension_zip, path_mock_extension_json_custom_display_name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2018. All Rights Reserved.

""" setup.py for resilient-circuits module """

from __future__ import print_function
import os
from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))

requires_resilient_version = "29.0"
major, minor = requires_resilient_version.split('.', 2)[:2]

setup(
    name='resilient_circuits',
    use_scm_version={"root": "../", "relative_to": __file__},
    setup_requires=['setuptools_scm'],
    url='https://developer.ibm.com/resilient',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Security',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],

    author='IBM Resilient',
    install_requires=[
        'stompest>=2.3.0',
        'requests>=2.6.0',
        'circuits',
        'pytz',
        'jinja2>=2.10.0',
        'pysocks',
        'filelock>=2.0.5',
        'setuptools>=41.0.0',
        'resilient>={}.{}'.format(major, minor)
    ],
    author_email='support@resilientsystems.com',
    description='Resilient Circuits Framework for Custom Integrations',
    long_description='Resilient Circuits Framework for Custom Integrations',
    packages=find_packages(),
    include_package_data=True,
    platforms='any',
    entry_points={
        'console_scripts': ['res-action-test = resilient_circuits.bin.res_action_test:main',
                            'resilient-circuits = resilient_circuits.bin.resilient_circuits_cmd:main']
    }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2019. All Rights Reserved.

""" Python Module that exposes the ExtException class """


class ExtException(Exception):
    """Custom Exception for ext commands"""

    # Class variable to track what resilient-circuits ext: command was run
    # This is set in the __init__ of the Ext class
    command_ran = ""

    def __init__(self, message):
        self.message = "\nresilient-circuits %s FAILED\nERROR: %s" % (ExtException.command_ran, message)

        # Call the base class
        super(ExtException, self).__init__(message)

    def __str__(self):
        return self.message



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2018. All Rights Reserved.

""" nosetests --with-coverage --cover-package=resilient_circuits """


def test():
    """The most basic tests"""
    from resilient_circuits import actions_component
    from resilient_circuits import app
    from resilient_circuits import component_loader
    from resilient_circuits import keyring_arguments
    from resilient_circuits import rest_helper
    from resilient_circuits import template_functions
    pass
